# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Time Entry views
"""

from rattail_harvest.db.model import HarvestCacheTimeEntry
from rattail_harvest.harvest.config import get_harvest_url

from .master import HarvestMasterView


class HarvestCacheTimeEntryView(HarvestMasterView):
    """
    Master view for Harvest Time Entries
    """
    model_class = HarvestCacheTimeEntry
    url_prefix = '/harvest/time-entries'
    route_prefix = 'harvest.time_entries'

    grid_columns = [
        'id',
        'spent_date',
        'user',
        'client',
        'project',
        'task',
        'hours',
        'notes',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.set_type('hours', 'duration_hours')

        g.set_sort_defaults('spent_date', 'desc')

        g.set_link('id')
        g.set_link('user')
        g.set_link('client')
        g.set_link('notes')

    def configure_form(self, f):
        super().configure_form(f)

        # make sure id is first field
        f.remove('id')
        f.insert(0, 'id')

        # user
        f.remove('user_id')
        f.set_renderer('user', self.render_harvest_user)

        # client
        f.remove('client_id')
        f.set_renderer('client', self.render_harvest_client)

        # project
        f.remove('project_id')
        f.set_renderer('project', self.render_harvest_project)

        # task
        f.remove('task_id')
        f.set_renderer('task', self.render_harvest_task)

        # hours
        f.set_renderer('hours', self.render_hours)

        f.set_type('notes', 'text')

        f.set_type('billable_rate', 'currency')
        f.set_type('cost_rate', 'currency')

    def render_hours(self, entry, field):
        hours = getattr(entry, field)
        app = self.get_rattail_app()
        duration = app.render_duration(hours=hours)
        return f"{hours} ({duration})"

    def get_xref_buttons(self, entry):
        buttons = super().get_xref_buttons(entry)
        model = self.model

        # harvest
        url = get_harvest_url(self.rattail_config)
        if url:
            url = '{}/time/day/{}/{}'.format(
                url,
                entry.spent_date.strftime('%Y/%m/%d'),
                entry.user_id)
            buttons.append(self.make_xref_button(url=url,
                                                 text="View in Harvest"))

        return buttons

    def import_from_harvest(self):
        app = self.get_rattail_app()
        handler = app.get_import_handler('to_rattail.from_harvest.import', require=True)
        importer = handler.get_importer('HarvestCacheTimeEntry')
        importer.session = self.Session()
        importer.setup()

        cache_entry = self.get_instance()
        if self.oneoff_import(importer, local_object=cache_entry):
            self.request.session.flash(f"{self.get_model_title()} has been "
                                       f"(re-)imported from Harvest: {cache_entry}")
        else:
            self.request.session.flash("Import failed!", 'error')

        return self.redirect(self.get_action_url('view', cache_entry))

    @classmethod
    def defaults(cls, config):
        route_prefix = cls.get_route_prefix()
        instance_url_prefix = cls.get_instance_url_prefix()
        permission_prefix = cls.get_permission_prefix()
        model_title = cls.get_model_title()

        # normal defaults
        cls._defaults(config)

        # import from harvest
        config.add_tailbone_permission(permission_prefix,
                                       f'{permission_prefix}.import_from_harvest',
                                       f"Re-Import {model_title} from Harvest")
        config.add_route(f'{route_prefix}.import_from_harvest',
                         f'{instance_url_prefix}/import-from-harvest',
                         request_method='POST')
        config.add_view(cls, attr='import_from_harvest',
                        route_name=f'{route_prefix}.import_from_harvest',
                        permission=f'{permission_prefix}.import_from_harvest')


def defaults(config, **kwargs):
    base = globals()

    HarvestCacheTimeEntryView = kwargs.get('HarvestCacheTimeEntryView', base['HarvestCacheTimeEntryView'])
    HarvestCacheTimeEntryView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest User views
"""

from rattail_harvest.db.model import HarvestCacheUser
from rattail_harvest.harvest.config import get_harvest_url

import colander

from tailbone import forms
from .master import HarvestMasterView


class HarvestCacheUserView(HarvestMasterView):
    """
    Master view for Harvest Users
    """
    model_class = HarvestCacheUser
    url_prefix = '/harvest/users'
    route_prefix = 'harvest.users'

    labels = {
        'avatar_url': "Avatar URL",
    }

    grid_columns = [
        'id',
        'first_name',
        'last_name',
        'email',
        'telephone',
        'timezone',
        'is_admin',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)
        model = self.model

        g.set_joiner('person_name', lambda q: q.outerjoin(model.Person))
        g.set_filter('person_name', model.Person.display_name)

        g.set_sort_defaults('first_name')

        g.set_link('id')
        g.set_link('first_name')
        g.set_link('last_name')
        g.set_link('email')

    def configure_form(self, f):
        super().configure_form(f)
        model = self.model
        user = f.model_instance

        # person
        f.set_renderer('person', self.render_person)
        if self.creating or self.editing:
            if 'person' in f.fields:
                f.remove('person_uuid')
                f.replace('person', 'person_uuid')
                person_display = ""
                if self.request.method == 'POST':
                    if self.request.POST.get('person_uuid'):
                        person = self.Session.get(model.Person,
                                                  self.request.POST['person_uuid'])
                        if person:
                            person_display = str(person)
                elif self.editing:
                    person_display = str(user.person or '')
                people_url = self.request.route_url('people.autocomplete')
                f.set_widget('person_uuid', forms.widgets.JQueryAutocompleteWidget(
                    field_display=person_display, service_url=people_url))
                f.set_validator('person_uuid', self.valid_person)
                f.set_label('person_uuid', "Person")
        else:
            f.remove('person_uuid')

        f.set_type('weekly_capacity', 'duration')

        f.set_type('default_hourly_rate', 'currency')
        f.set_type('cost_rate', 'currency')

        f.set_renderer('avatar_url', self.render_url)

        # timestamps
        if self.creating or self.editing:
            f.remove('created_at')
            f.remove('updated_at')

        # time_entries
        # TODO: should add this as child rows/grid instead
        f.remove('time_entries')

    def valid_person(self, node, value):
        model = self.model
        if value:
            person = self.Session.get(model.Person, value)
            if not person:
                raise colander.Invalid(node, "Person not found (you must *select* a record)")

    def get_xref_buttons(self, user):
        buttons = super().get_xref_buttons(user)
        model = self.model

        # harvest proper
        url = get_harvest_url(self.rattail_config)
        if url:
            url = '{}/team'.format(url)
            buttons.append(self.make_xref_button(url=url, text="View in Harvest"))

        return buttons


def defaults(config, **kwargs):
    base = globals()

    HarvestCacheUserView = kwargs.get('HarvestCacheUserView', base['HarvestCacheUserView'])
    HarvestCacheUserView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest master view
"""

from rattail_harvest.db.model import HarvestCacheTimeEntry

from webhelpers2.html import tags

from tailbone.views import MasterView


class HarvestMasterView(MasterView):
    """
    Base class for Harvest master views
    """
    creatable = False
    touchable = True
    has_versions = True
    model_row_class = HarvestCacheTimeEntry

    labels = {
        'id': "ID",
        'user_id': "User ID",
        'client_id': "Client ID",
        'project_id': "Project ID",
        'task_id': "Task ID",
        'invoice_id': "Invoice ID",
    }

    row_labels = {
        'id': "ID",
    }

    def configure_form(self, f):
        super(HarvestMasterView, self).configure_form(f)
        f.remove('time_entries')

    def render_harvest_user(self, obj, field):
        user = getattr(obj, field)
        if user:
            text = str(user)
            url = self.request.route_url('harvest.users.view', uuid=user.uuid)
            return tags.link_to(text, url)

    def render_harvest_client(self, obj, field):
        client = getattr(obj, field)
        if client:
            text = str(client)
            url = self.request.route_url('harvest.clients.view', uuid=client.uuid)
            return tags.link_to(text, url)

    def render_harvest_project(self, obj, field):
        project = getattr(obj, field)
        if project:
            text = str(project)
            url = self.request.route_url('harvest.projects.view', uuid=project.uuid)
            return tags.link_to(text, url)

    def render_harvest_task(self, obj, field):
        task = getattr(obj, field)
        if task:
            text = str(task)
            url = self.request.route_url('harvest.tasks.view', uuid=task.uuid)
            return tags.link_to(text, url)

    def configure_row_grid(self, g):
        super(HarvestMasterView, self).configure_row_grid(g)
        g.set_sort_defaults('spent_date', 'desc')

    def row_view_action_url(self, entry, i):
        return self.request.route_url('harvest.time_entries.view', uuid=entry.uuid)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Client views
"""

from rattail_harvest.db.model import HarvestCacheClient
from rattail_harvest.harvest.config import get_harvest_url

from webhelpers2.html import HTML, tags

from .master import HarvestMasterView


class HarvestCacheClientView(HarvestMasterView):
    """
    Master view for Harvest Clients
    """
    model_class = HarvestCacheClient
    url_prefix = '/harvest/clients'
    route_prefix = 'harvest.clients'

    grid_columns = [
        'id',
        'name',
        'currency',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)

        g.filters['name'].default_active = True
        g.filters['name'].default_verb = 'contains'

        g.filters['is_active'].default_active = True
        g.filters['is_active'].default_verb = 'is_true'

        g.set_sort_defaults('name')

        g.set_link('id')
        g.set_link('name')

    def grid_extra_class(self, client, i):
        if not client.is_active:
            return 'warning'

    def configure_form(self, f):
        super().configure_form(f)

        # projects
        f.set_renderer('projects', self.render_projects)

        # time_entries
        f.remove_field('time_entries')

    def render_projects(self, client, field):
        projects = client.projects
        if not projects:
            return

        items = []
        for project in projects:
            text = "({}) {}".format(project.code, project.name)
            url = self.request.route_url('harvest.projects.view', uuid=project.uuid)
            items.append(HTML.tag('li', c=[tags.link_to(text, url)]))
        return HTML.tag('ul', c=items)

    def get_xref_buttons(self, client):
        buttons = super().get_xref_buttons(client)
        model = self.model

        # harvest proper
        url = get_harvest_url(self.rattail_config)
        if url:
            url = '{}/clients'.format(url)
            buttons.append(self.make_xref_button(url=url,
                                                 text="View in Harvest"))

        return buttons


def defaults(config, **kwargs):
    base = globals()

    HarvestCacheClientView = kwargs.get('HarvestCacheClientView', base['HarvestCacheClientView'])
    HarvestCacheClientView.defaults(config)


def includeme(config):
    defaults(config)



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2022 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Common menus for Harvest
"""


def make_harvest_menu(request):
    url = request.route_url

    harvest_menu = {
        'title': "Harvest",
        'type': 'menu',
        'items': [
            {
                'title': "Users",
                'url': url('harvest.users'),
                'perm': 'harvest.users.list',
            },
            {
                'title': "Clients",
                'url': url('harvest.clients'),
                'perm': 'harvest.clients.list',
            },
            {
                'title': "Projects",
                'url': url('harvest.projects'),
                'perm': 'harvest.projects.list',
            },
            {
                'title': "Tasks",
                'url': url('harvest.tasks'),
                'perm': 'harvest.tasks.list',
            },
            {
                'title': "Time Entries",
                'url': url('harvest.time_entries'),
                'perm': 'harvest.time_entries.list',
            },
        ],
    }

    harvest_url = request.rattail_config.get('harvest', 'url')
    if harvest_url:
        harvest_menu['items'].insert(
            0, {
                'title': "Go to Harvest",
                'url': harvest_url,
                'target': '_blank',
            })
        harvest_menu['items'].insert(
            1, {'type': 'sep'})

    return harvest_menu



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2024 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Tasks for tailbone-harvest
"""

import os
import re
import shutil

from invoke import task


here = os.path.abspath(os.path.dirname(__file__))
__version__ = None
pattern = re.compile(r'^version = "(\d+\.\d+\.\d+)"$')
with open(os.path.join(here, 'pyproject.toml'), 'rt') as f:
    for line in f:
        line = line.rstrip('\n')
        match = pattern.match(line)
        if match:
            __version__ = match.group(1)
            break
if not __version__:
    raise RuntimeError("could not parse version!")


@task
def release(c):
    """
    Release a new version of tailbone-harvest
    """
    # rebuild local tar.gz file for distribution
    if os.path.exists('tailbone_harvest.egg-info'):
        shutil.rmtree('tailbone_harvest.egg-info')
    c.run('python -m build --sdist')

    # upload to public PyPI
    filename = f'tailbone_harvest-{__version__}.tar.gz'
    c.run(f'twine upload dist/{filename}')



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2023 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Harvest Project views
"""

from rattail_harvest.db.model import HarvestCacheProject
from rattail_harvest.harvest.config import get_harvest_url

from .master import HarvestMasterView


class HarvestCacheProjectView(HarvestMasterView):
    """
    Master view for Harvest Projects
    """
    model_class = HarvestCacheProject
    url_prefix = '/harvest/projects'
    route_prefix = 'harvest.projects'

    has_rows = True

    grid_columns = [
        'id',
        'client',
        'name',
        'code',
        'is_active',
        'is_billable',
        'bill_by',
        'hourly_rate',
        'fee',
    ]

    row_grid_columns = [
        'id',
        'spent_date',
        'user',
        'client',
        'task',
        'hours',
    ]

    def configure_grid(self, g):
        super().configure_grid(g)
        model = self.model

        g.set_joiner('client', lambda q: q.outerjoin(model.HarvestCacheClient))
        g.set_sorter('client', model.HarvestCacheClient.name)
        g.set_filter('client', model.HarvestCacheClient.name, label="Client Name")
        g.filters['client'].default_active = True
        g.filters['client'].default_verb = 'contains'

        g.filters['is_active'].default_active = True
        g.filters['is_active'].default_verb = 'is_true'

        g.set_type('hourly_rate', 'currency')
        g.set_type('fee', 'currency')

        g.set_sort_defaults('client')

        g.set_filters_sequence([
            'id',
            'name',
            'client',
        ])

        g.set_link('id')
        g.set_link('client')
        g.set_link('name')
        g.set_link('code')

    def grid_extra_class(self, project, i):
        if not project.is_active:
            return 'warning'

    def configure_form(self, f):
        super().configure_form(f)

        f.set_type('hourly_rate', 'currency')

        if self.editing:
            f.remove('client')
            f.set_type('over_budget_notification_date', 'date_jquery')
            f.set_type('starts_on', 'date_jquery')
            f.set_type('ends_on', 'date_jquery')
            f.set_readonly('created_at')
            f.set_readonly('updated_at')

    def get_xref_buttons(self, project):
        buttons = super().get_xref_buttons(project)
        model = self.model

        # harvest
        url = get_harvest_url(self.rattail_config)
        if url:
            url = '{}/projects/{}'.format(url, project.id)
            buttons.append(self.make_xref_button(url=url,
                                                 text="View in Harvest"))

        return buttons

    def get_row_data(self, project):
        model = self.model
        return self.Session.query(model.HarvestCacheTimeEntry)\
                           .filter(model.HarvestCacheTimeEntry.project == project)

    def get_parent(self, entry):
        return entry.project


def defaults(config, **kwargs):
    base = globals()

    HarvestCacheProjectView = kwargs.get('HarvestCacheProjectView', base['HarvestCacheProjectView'])
    HarvestCacheProjectView.defaults(config)


def includeme(config):
    defaults(config)
